﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Linq.Expressions;
using IdeaBlade.EntityModel;
using System.Collections;
using IdeaBlade.Linq;
using System.Diagnostics;
using System.Reflection;

/// <summary>
/// ????? Section on serializing entities was removed; Jay says they now serialize
/// ????? like any other .NET object. Be sure to remove the discussion from the chapter.
/// </summary>
namespace SilverlightConsole {
  public class MainPageViewModel : INotifyPropertyChanged {
    
    #region Start

    public MainPageViewModel() {
      //if (!System.ComponentModel.DesignerProperties.IsInDesignTool) {
      //  RunDemos();
      //}
    }

    public void RunDemos() {
      QueueDemos();
      ExecuteQueue();
    }

    #endregion Start

    #region Select Demos to Run

    /// <summary>
    /// This method populates a queue of demos. 
    /// </summary>
    /// <remarks>
    /// Comment out all demos except those you wish to run. You can run
    /// them all in rapid-fire sequence, but be aware that some produce 
    /// output asynchronously, so that the output can becomes somewhat 
    /// interlaced. However, all lines of output are prefixed with the
    /// name of the sample method with which they are associated.
    /// </remarks>
    private void QueueDemos() {
      IncludeBasicQueriesDemo();
    }

    private void IncludeBasicQueriesDemo() {
      MainDemo aMainDemo = new MainDemo(OutputWriter);
      _demos.Enqueue(() => aMainDemo.BasicQuerySyntaxQuery());
    }

    #endregion Select Demos to Run

    #region Data Bound Properties

    public virtual string Output {
      get {
        return _output;
      }
      set {
        _output = value;
        RaisePropertyChanged("Output");
      }
    }

    //public Employee CurrentEmployee {
    //  get { return _currentEmployee; }
    //  set {
    //    SetCurrentEmployee(value);
    //  }
    //}

    //private void SetCurrentEmployee(Employee employee) {
    //  _currentEmployee = employee;
    //  RaisePropertyChanged("CurrentEmployee");
    //  if (null == CurrentEmployee)
    //    return;

    //  Log("Now viewing Employee " + CurrentEmployee.LastName + ", " + CurrentEmployee.FirstName);
    //  GetCurrentEmployeeOrders();
    //}

    //public ObservableCollection<Customer> Customers {
    //  get { return _customers; }
    //}

    //public ObservableCollection<Employee> Employees {
    //  get { return _employees; }
    //}

    //public ObservableCollection<string> StatusMessages {
    //  get { return _statusMessages; }
    //}

    #endregion Data Bound Properties

    #region INotifyPropertyChanged

    public event PropertyChangedEventHandler PropertyChanged = delegate { };

    protected void RaisePropertyChanged(string propertyName) {
      PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    #endregion

    #region Utility Methods

    public void OutputWriter(string output) {
      _outputStringBuilder.Append(output);
      this.Output = _outputStringBuilder.ToString();
      ExecuteQueue();
    }

    private void ExecuteQueue() {
      if (_demos.Count > 0) {
        var demoLaunchMethod = _demos.Dequeue();
        demoLaunchMethod();
      }
    }

    #endregion Utility Methods

    #region Private Fields

    private Queue<Action> _demos = new Queue<Action>();
    private StringBuilder _outputStringBuilder = new StringBuilder();
    private string _output = "";

    #endregion Private Fields

  }
}
