﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using IdeaBlade.EntityModel;

namespace SilverlightConsole {

  public delegate void ResultsReporter(string output);

  public static class Utility {

    public static void ResetEntityManager(EntityManager em) {
      em.Clear();
      //em.UseAsyncNavigation = false;  //this property is read-only in Silverlight apps
    }

    
    public static string DisplayCacheContents(NorthwindIBModelManager em, string headerMessage) {
      StringBuilder output = new StringBuilder(headerMessage + "\n");
      output.Append(String.Format("\tCustomers: {0}", em.Customers.With(QueryStrategy.CacheOnly).Count()) + "\n");
      output.Append(String.Format("\tOrders: {0}", em.Orders.With(QueryStrategy.CacheOnly).Count()) + "\n");
      output.Append(String.Format("\tOrderDetails: {0}", em.OrderDetails.With(QueryStrategy.CacheOnly).Count() + "\n"));
      output.Append(String.Format("\tProducts: {0}", em.Products.With(QueryStrategy.CacheOnly).Count() + "\n"));
      output.Append(String.Format("\tSuppliers: {0}", em.Suppliers.With(QueryStrategy.CacheOnly).Count() + "\n"));
      output.Append(String.Format("\tEmployees: {0}", em.Employees.With(QueryStrategy.CacheOnly).Count() + "\n"));
      output.Append(String.Format("\tEmployeeTerritories: {0}", em.EmployeeTerritories.With(QueryStrategy.CacheOnly).Count() + "\n"));
      return output.ToString();
    }

    public static string DisplayCacheContents(NorthwindIBModelManager em) {
      return DisplayCacheContents(em, "Contents of Cache:");
    }

    public static string GetMethodOutputHeader(string methodName) {
      string underline = new String('=', methodName.Length + 2);
      return String.Format("{0}\n{1}()\n{2}\n", underline, methodName, underline);
    }

    public static string NoOutputMessage {
      get {
        return "This method produces no signficant output: simply inspect the source code for syntax models.";
      }
    }

    public static void FlushOutputBuffer(StringBuilder localOutput, ResultsReporter reporter) {
      string output = localOutput.ToString();
      localOutput.Clear();  
      
      // It's crucial to clear the localOutput buffer before calling the reporter callback method.
      // The latter gives the order to execute the next sample in the sequence, and that
      // sample immediately begins adding output to the same buffer.
      reporter(output);
    }

    public static void InitializeDemo(
      string nameOfDemo,
      ref ResultsReporter callbackMethod,
      ref ResultsReporter localCallbackDelegateInstance,
      ref StringBuilder localOutputStringBuilder
    ) {
      localCallbackDelegateInstance = callbackMethod;
      localOutputStringBuilder = new StringBuilder();
      localOutputStringBuilder.Append(String.Format("\n>>> Initializing [{0}]...\n", nameOfDemo));
    }

  }
}
